<?php
/**
 * @package     CSVI
 * @subpackage  SEF
 *
 * @author      RolandD Cyber Produksi <contact@csvimproved.com>
 * @copyright   Copyright (C) 2006 - @year@ RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://csvimproved.com
 */

defined('_JEXEC') or die;

/**
 * SEF helper class for the component.
 *
 * @package     CSVI
 * @subpackage  SEF
 * @since       4.0
 */
class CsviHelperSef
{
	/**
	 * The domain name to use for URLs
	 *
	 * @var    string
	 * @since  4.0
	 */
	private $domainname = null;

	/**
	 * An instance of the CsviHelperTemplate.
	 *
	 * @var    CsviHelperTemplate
	 * @since  6.0
	 */
	private $template = null;

	/**
	 * An instance of the CsviHelperLog.
	 *
	 * @var    CsviHelperLog
	 * @since  6.0
	 */
	private $log = null;

	/**
	 * JDatabaseDriver
	 *
	 * @var    JDatabaseDriver
	 * @since  7.1.0
	 */
	private $db;

	/**
	 * Total number of SEF URLs in the database
	 *
	 * @var    int
	 * @since  7.1.0
	 */
	private $sefCount = 0;

	/**
	 * Constructor.
	 *
	 * @param   CsviHelperSettings  $settings  An instance of CsviHelperSettings.
	 * @param   CsviHelperTemplate  $template  An instance of CsviHelperTemplate.
	 * @param   CsviHelperLog       $log       An instance of CsviHelperLog.
	 *
	 * @since   4.0
	 *
	 * @throws  CsviException
	 */
	public function __construct(CsviHelperSettings $settings, CsviHelperTemplate $template, CsviHelperLog $log)
	{
		$this->domainname = $settings->get('hostname');

		// Make sure we have a valid domain name
		if (filter_var($this->domainname, FILTER_VALIDATE_URL) === false)
		{
			throw new CsviException(JText::_('COM_CSVI_NO_VALID_DOMAIN_NAME_SET'));
		}

		$this->template   = $template;
		$this->log        = $log;
		$this->db         = JFactory::getDbo();

		if ($template->get('emptysefcache', false))
		{
			$this->db->truncateTable('#__csvi_sefurls');
		}

		// Check if there are any SEF URLs in the database
		$this->countSefUrls();
	}

	/**
	 * Count the total number of SEF URLs.
	 *
	 * @return  void
	 *
	 * @since   7.1.0
	 */
	private function countSefUrls()
	{
		$query = $this->db->getQuery(true)
			->select('COUNT(' . $this->db->quoteName('sefurl_id') . ')')
			->from($this->db->quoteName('#__csvi_sefurls'));
		$this->db->setQuery($query);

		$this->sefCount = $this->db->loadResult();
	}

	/**
	 * Create a SEF URL by querying the URL.
	 *
	 * @param   string  $url  The URL to change to SEF.
	 *
	 * @return  string  The SEF URL.
	 *
	 * @since   6.0
	 */
	public function getSefUrl($url)
	{
		if ($this->template->get('exportsef', false))
		{
			if ($this->sefCount > 0)
			{
				$sefUrl = $this->loadSefUrl($url);

				if ($sefUrl)
				{
					return $sefUrl;
				}
			}

			$parseUrl = base64_encode($url);
			$language = substr($this->template->get('language'), 0, 2);
			$language = $language ? '&lang=' . $language : '';
			$http     = JHttpFactory::getHttp(null, array('curl', 'stream'));
			$result   = $http->get($this->domainname . '/index.php?option=com_csvi&task=sefs.getsef&parseurl=' . $parseUrl . '&format=json' . $language);
			$output   = json_decode($result->body);

			if (is_object($output) && $output->success)
			{
				if ($this->template->get('cachesefurls', false))
				{
					$this->saveSefUrl($url, $output->data);
				}

				return $output->data;
			}
		}

		// Get position of the forward slash
		$slashPosition = strpos($url, '/');

		if ($slashPosition > 0 || $slashPosition === false)
		{
			$url = '/' . $url;
		}

		return $this->domainname . $url;
	}

	/**
	 * Load SEF URL from database.
	 *
	 * @param   string  $url  The URL to change to SEF.
	 *
	 * @return  string  The SEF URL from the database or empty if nothing is found.
	 *
	 * @since   7.1.0
	 */
	private function loadSefUrl($url)
	{
		$query = $this->db->getQuery(true)
			->select($this->db->quoteName('sefurl'))
			->from($this->db->quoteName('#__csvi_sefurls'))
			->where($this->db->quoteName('plainurl') . ' = ' . $this->db->quote($url));

		$this->db->setQuery($query);

		return $this->db->loadResult();
	}

	/**
	 * Store a SEF URL in the database.
	 *
	 * @param   string  $plainUrl  The non-SEF URL to store.
	 * @param   string  $sefUrl    The SEF URL to store.
	 *
	 * @return  void
	 *
	 * @since   7.1.0
	 */
	private function saveSefUrl($plainUrl, $sefUrl)
	{
		$query = $this->db->getQuery(true)
			->insert($this->db->quoteName('#__csvi_sefurls'))
			->columns(
				array(
					'plainurl',
					'sefurl',
				)
			)
			->values(
				$this->db->quote($plainUrl) . ',' . $this->db->quote($sefUrl)
			);
		$this->db->setQuery($query)->execute();
	}
}
